@@ -18,7 +18,14 @@
  */
 
 #import <Foundation/Foundation.h>
+#import <Cordova/CDVAvailabilityDeprecated.h>
 
+/**
+ @Metadata {
+    @Available(Cordova, introduced: "2.3.0", deprecated: "8.0.0")
+ }
+ */
+CDV_DEPRECATED(8.0.0, "")
 @protocol CDVScreenOrientationDelegate <NSObject>
 
 - (UIInterfaceOrientationMask)supportedInterfaceOrientations;
