@@ -24,11 +24,11 @@ import PackageDescription
 let package = Package(
     name: "Cordova",
     platforms: [
-        .iOS(.v11),
+        .iOS(.v13),
         .macCatalyst(.v13)
     ],
     products: [
-        .library(name: "CordovaLib", targets: ["Cordova"])
+        .library(name: "Cordova", targets: ["Cordova"])
     ],
     dependencies: [],
     targets: [
