@@ -17,13 +17,12 @@
     under the License.
 */
 
-/* jslint node: true */
+/* eslint no-useless-escape : 0 */
 
 const VERSION = require('../package.json').version;
 
 const fs = require('node:fs');
 const path = require('node:path');
-const util = require('node:util');
 const projectFile = require('./projectFile');
 const check_reqs = require('./check_reqs');
 const {
@@ -34,6 +33,12 @@ const {
     PluginManager
 } = require('cordova-common');
 
+// check the value is true or 'true' or 'TRUE' or 1
+const _isTrue = (value) => {
+    return (value === true) ||
+    (typeof value === 'string' && value.toLowerCase() === 'true') || value === 1;
+};
+
 function setupEvents (externalEventEmitter) {
     if (externalEventEmitter) {
         // This will make the platform internal events visible outside
@@ -81,39 +86,20 @@ class Api {
 
         setupEvents(events);
 
-        let xcodeProjDir;
-        let xcodeCordovaProj;
-
-        try {
-            const xcodeProjDir_array = fs.readdirSync(this.root).filter(e => e.match(/\.xcodeproj$/i));
-            if (xcodeProjDir_array.length > 1) {
-                for (let x = 0; x < xcodeProjDir_array.length; x++) {
-                    if (xcodeProjDir_array[x].substring(0, 2) === '._') {
-                        xcodeProjDir_array.splice(x, 1);
-                    }
-                }
-            }
-            xcodeProjDir = xcodeProjDir_array[0];
-
-            if (!xcodeProjDir) {
-                throw new CordovaError(`The provided path "${this.root}" is not a Cordova iOS project.`);
-            }
-
-            const cordovaProjName = xcodeProjDir.substring(xcodeProjDir.lastIndexOf(path.sep) + 1, xcodeProjDir.indexOf('.xcodeproj'));
-            xcodeCordovaProj = path.join(this.root, cordovaProjName);
-        } catch (e) {
-            throw new CordovaError(`The provided path "${this.root}" is not a Cordova iOS project.`);
+        const xcodeProjDir = path.join(this.root, 'App.xcodeproj');
+        if (!fs.existsSync(xcodeProjDir)) {
+            throw new CordovaError(`The provided path "${this.root}" is not an up-to-date Cordova iOS project.`);
         }
 
         this.locations = {
             root: this.root,
             www: path.join(this.root, 'www'),
             platformWww: path.join(this.root, 'platform_www'),
-            configXml: path.join(xcodeCordovaProj, 'config.xml'),
+            configXml: path.join(this.root, 'App', 'config.xml'),
             defaultConfigXml: path.join(this.root, 'cordova', 'defaults.xml'),
-            pbxproj: path.join(this.root, xcodeProjDir, 'project.pbxproj'),
-            xcodeProjDir: path.join(this.root, xcodeProjDir),
-            xcodeCordovaProj
+            pbxproj: path.join(xcodeProjDir, 'project.pbxproj'),
+            xcodeProjDir,
+            xcodeCordovaProj: path.join(this.root, 'App')
         };
     }
 
@@ -245,6 +231,7 @@ class Api {
      */
     addPlugin (plugin, installOptions) {
         const xcodeproj = projectFile.parse(this.locations);
+        const { SwiftPackage, isSwiftPackagePlugin } = require('./SwiftPackage');
 
         installOptions = installOptions || {};
         installOptions.variables = installOptions.variables || {};
@@ -256,7 +243,13 @@ class Api {
         return PluginManager.get(this.platform, this.locations, xcodeproj)
             .addPlugin(plugin, installOptions)
             .then(() => {
-                if (plugin != null) {
+                if (plugin != null && isSwiftPackagePlugin(plugin)) {
+                    const spm = new SwiftPackage(this.locations.root);
+                    spm.addPlugin(plugin, installOptions);
+                }
+            })
+            .then(() => {
+                if (plugin != null && !isSwiftPackagePlugin(plugin)) {
                     const headerTags = plugin.getHeaderFiles(this.platform);
                     const bridgingHeaders = headerTags.filter(obj => obj.type === 'BridgingHeader');
                     if (bridgingHeaders.length > 0) {
@@ -298,11 +291,18 @@ class Api {
      */
     removePlugin (plugin, uninstallOptions) {
         const xcodeproj = projectFile.parse(this.locations);
+        const { SwiftPackage, isSwiftPackagePlugin } = require('./SwiftPackage');
 
         return PluginManager.get(this.platform, this.locations, xcodeproj)
             .removePlugin(plugin, uninstallOptions)
             .then(() => {
-                if (plugin != null) {
+                if (plugin != null && isSwiftPackagePlugin(plugin)) {
+                    const spm = new SwiftPackage(this.locations.root);
+                    spm.removePlugin(plugin);
+                }
+            })
+            .then(() => {
+                if (plugin != null && !isSwiftPackagePlugin(plugin)) {
                     const headerTags = plugin.getHeaderFiles(this.platform);
                     const bridgingHeaders = headerTags.filter(obj => obj.type === 'BridgingHeader');
                     if (bridgingHeaders.length > 0) {
@@ -338,6 +338,11 @@ class Api {
      * @return  {Promise}  Return a promise
      */
     addPodSpecs (plugin, podSpecs, installOptions) {
+        if (!podSpecs.length) {
+            return;
+        }
+        const { isSwiftPackagePlugin } = require('./SwiftPackage');
+
         const project_dir = this.locations.root;
         const project_name = this.locations.xcodeCordovaProj.split(path.sep).pop();
         const minDeploymentTarget = this.getPlatformInfo().projectConfig.getPreference('deployment-target', 'ios');
@@ -347,50 +352,49 @@ class Api {
         const podsjsonFile = new PodsJson(path.join(project_dir, PodsJson.FILENAME));
         const podfileFile = new Podfile(path.join(project_dir, Podfile.FILENAME), project_name, minDeploymentTarget);
 
-        if (podSpecs.length) {
-            events.emit('verbose', 'Adding pods since the plugin contained <podspecs>');
-            podSpecs.forEach(obj => {
-                // declarations
-                if (obj.declarations) {
-                    Object.keys(obj.declarations).forEach(key => {
-                        if (obj.declarations[key] === 'true') {
-                            const declaration = Podfile.proofDeclaration(key);
-                            const podJson = {
-                                declaration
-                            };
-                            const val = podsjsonFile.getDeclaration(declaration);
-                            if (val) {
-                                podsjsonFile.incrementDeclaration(declaration);
-                            } else {
-                                podJson.count = 1;
-                                podsjsonFile.setJsonDeclaration(declaration, podJson);
-                                podfileFile.addDeclaration(podJson.declaration);
-                            }
-                        }
-                    });
-                }
-
-                // sources
-                if (obj.sources) {
-                    Object.keys(obj.sources).forEach(key => {
+        events.emit('verbose', 'Adding pods since the plugin contained <podspecs>');
+        podSpecs.forEach(obj => {
+            // declarations
+            if (obj.declarations) {
+                Object.keys(obj.declarations).forEach(key => {
+                    if (obj.declarations[key] === 'true') {
+                        const declaration = Podfile.proofDeclaration(key);
                         const podJson = {
-                            source: obj.sources[key].source
+                            declaration
                         };
-                        const val = podsjsonFile.getSource(key);
+                        const val = podsjsonFile.getDeclaration(declaration);
                         if (val) {
-                            podsjsonFile.incrementSource(key);
+                            podsjsonFile.incrementDeclaration(declaration);
                         } else {
                             podJson.count = 1;
-                            podsjsonFile.setJsonSource(key, podJson);
-                            podfileFile.addSource(podJson.source);
+                            podsjsonFile.setJsonDeclaration(declaration, podJson);
+                            podfileFile.addDeclaration(podJson.declaration);
                         }
-                    });
-                }
+                    }
+                });
+            }
 
-                // libraries
-                if (obj.libraries) {
-                    Object.keys(obj.libraries).forEach(key => {
-                        let podJson = Object.assign({}, obj.libraries[key]);
+            // sources
+            if (obj.sources) {
+                Object.keys(obj.sources).forEach(key => {
+                    const podJson = {
+                        source: obj.sources[key].source
+                    };
+                    const val = podsjsonFile.getSource(key);
+                    if (val) {
+                        podsjsonFile.incrementSource(key);
+                    } else {
+                        podJson.count = 1;
+                        podsjsonFile.setJsonSource(key, podJson);
+                        podfileFile.addSource(podJson.source);
+                    }
+                });
+            }
+            if (obj.libraries) {
+                const isSPM = isSwiftPackagePlugin(plugin);
+                Object.keys(obj.libraries).forEach(key => {
+                    let podJson = Object.assign({}, obj.libraries[key]);
+                    if (!isSPM || (isSPM && !_isTrue(podJson.nospm))) {
                         podJson = replacePodSpecVariables(podJson, installOptions);
                         const val = podsjsonFile.getLibrary(key);
                         if (val) {
@@ -401,26 +405,25 @@ class Api {
                             podsjsonFile.setJsonLibrary(key, podJson);
                             podfileFile.addSpec(podJson.name, podJson);
                         }
-                    });
-                }
-            });
-
-            // now that all the pods have been processed, write to pods.json
-            podsjsonFile.write();
-
-            // only write and pod install if the Podfile changed
-            if (podfileFile.isDirty()) {
-                podfileFile.write();
-                events.emit('verbose', 'Running `pod install` (to install plugins)');
-                projectFile.purgeProjectFileCache(this.locations.root);
-
-                return podfileFile.install(check_reqs.check_cocoapods)
-                    .then(() => podsjsonFile.setSwiftVersionForCocoaPodsLibraries(this.root));
-            } else {
-                events.emit('verbose', 'Podfile unchanged, skipping `pod install`');
+                    }
+                });
             }
+        });
+
+        // now that all the pods have been processed, write to pods.json
+        podsjsonFile.write();
+
+        // only write and pod install if the Podfile changed
+        if (podfileFile.isDirty()) {
+            podfileFile.write();
+            events.emit('verbose', 'Running `pod install` (to install plugins)');
+            projectFile.purgeProjectFileCache(this.locations.root);
+
+            return podfileFile.install(check_reqs.check_cocoapods)
+                .then(() => podsjsonFile.setSwiftVersionForCocoaPodsLibraries(this.root));
+        } else {
+            events.emit('verbose', 'Podfile unchanged, skipping `pod install`');
         }
-        return Promise.resolve();
     }
 
     /**
@@ -433,6 +436,8 @@ class Api {
      */
 
     removePodSpecs (plugin, podSpecs, uninstallOptions) {
+        const { isSwiftPackagePlugin } = require('./SwiftPackage');
+
         const project_dir = this.locations.root;
         const project_name = this.locations.xcodeCordovaProj.split(path.sep).pop();
 
@@ -455,7 +460,7 @@ class Api {
                         if (val) {
                             podsjsonFile.decrementDeclaration(declaration);
                         } else {
-                            const message = util.format('plugin \"%s\" declaration \"%s\" does not seem to be in pods.json, nothing to remove. Will attempt to remove from Podfile.', plugin.id, podJson.declaration); /* eslint no-useless-escape : 0 */
+                            const message = `plugin "${plugin.id}" declaration "${podJson.declaration}" does not seem to be in pods.json, nothing to remove. Will attempt to remove from Podfile.`;
                             events.emit('verbose', message);
                         }
                         if (!val || val.count === 0) {
@@ -472,7 +477,7 @@ class Api {
                     if (val) {
                         podsjsonFile.decrementSource(key);
                     } else {
-                        const message = util.format('plugin \"%s\" source \"%s\" does not seem to be in pods.json, nothing to remove. Will attempt to remove from Podfile.', plugin.id, podJson.source); /* eslint no-useless-escape : 0 */
+                        const message = `plugin "${plugin.id}" source "${podJson.source}" does not seem to be in pods.json, nothing to remove. Will attempt to remove from Podfile.`;
                         events.emit('verbose', message);
                     }
                     if (!val || val.count === 0) {
@@ -480,18 +485,21 @@ class Api {
                     }
                 });
                 // libraries
+                const isSPM = isSwiftPackagePlugin(plugin);
                 Object.keys(obj.libraries).forEach(key => {
                     let podJson = Object.assign({}, obj.libraries[key]);
-                    podJson = replacePodSpecVariables(podJson, uninstallOptions);
-                    const val = podsjsonFile.getLibrary(key);
-                    if (val) {
-                        podsjsonFile.decrementLibrary(key);
-                    } else {
-                        const message = util.format('plugin \"%s\" podspec \"%s\" does not seem to be in pods.json, nothing to remove. Will attempt to remove from Podfile.', plugin.id, podJson.name); /* eslint no-useless-escape : 0 */
-                        events.emit('verbose', message);
-                    }
-                    if (!val || val.count === 0) {
-                        podfileFile.removeSpec(podJson.name);
+                    if (!isSPM || (isSPM && !_isTrue(podJson.nospm))) {
+                        podJson = replacePodSpecVariables(podJson, uninstallOptions);
+                        const val = podsjsonFile.getLibrary(key);
+                        if (val) {
+                            podsjsonFile.decrementLibrary(key);
+                        } else {
+                            const message = `plugin "${plugin.id}" podspec "${podJson.name}" does not seem to be in pods.json, nothing to remove. Will attempt to remove from Podfile.`;
+                            events.emit('verbose', message);
+                        }
+                        if (!val || val.count === 0) {
+                            podfileFile.removeSpec(podJson.name);
+                        }
                     }
                 });
             });
