@@ -19,7 +19,6 @@
 
 const fs = require('node:fs');
 const path = require('node:path');
-const util = require('node:util');
 const xcode = require('xcode');
 const check_reqs = require('./check_reqs');
 const events = require('cordova-common').events;
@@ -37,11 +36,11 @@ function PodsJson (podsJsonPath) {
 
     const filename = this.path.split(path.sep).pop();
     if (filename !== PodsJson.FILENAME) {
-        throw new CordovaError(util.format('PodsJson: The file at %s is not `%s`.', this.path, PodsJson.FILENAME));
+        throw new CordovaError(`PodsJson: The file at ${this.path} is not \`${PodsJson.FILENAME}\`.`);
     }
 
     if (!fs.existsSync(this.path)) {
-        events.emit('verbose', util.format('pods.json: The file at %s does not exist.', this.path));
+        events.emit('verbose', `pods.json: The file at ${this.path} does not exist.`);
         events.emit('verbose', 'Creating new pods.json in platforms/ios');
         this.clear();
         this.write();
@@ -101,7 +100,7 @@ PodsJson.prototype.__remove = function (kind, name) {
     if (this.contents[kind][name]) {
         delete this.contents[kind][name];
         this.__dirty = true;
-        events.emit('verbose', util.format('Remove from pods.json for `%s` - `%s`', name));
+        events.emit('verbose', `Remove from pods.json for \`${name}\` - \`%s\``);
     }
 };
 
@@ -128,7 +127,7 @@ PodsJson.prototype.clear = function () {
 
 PodsJson.prototype.destroy = function () {
     fs.unlinkSync(this.path);
-    events.emit('verbose', util.format('Deleted `%s`', this.path));
+    events.emit('verbose', `Deleted \`${this.path}\``);
 };
 
 PodsJson.prototype.write = function () {
@@ -183,7 +182,7 @@ PodsJson.prototype.decrementDeclaration = function (name) {
 PodsJson.prototype.__setJson = function (kind, name, json) {
     this.contents[kind][name] = Object.assign({}, json);
     this.__dirty = true;
-    events.emit('verbose', util.format('Set pods.json for `%s` - `%s`', kind, name));
+    events.emit('verbose', `Set pods.json for \`${kind}\` - \`${name}\``);
 };
 
 // sample json for library
