@@ -18,9 +18,8 @@
  */
 
 const fs = require('node:fs');
-const fsp = require('node:fs/promises');
 const path = require('node:path');
-const util = require('node:util');
+const nopt = require('nopt');
 const which = require('which');
 const execa = require('execa');
 const { CordovaError, events } = require('cordova-common');
@@ -61,12 +60,11 @@ const buildFlagMatchers = {
  * a project path and name
  *
  * @param {*} projectPath
- * @param {*} projectName
  */
-function createProjectObject (projectPath, projectName) {
+function createProjectObject (projectPath) {
     const locations = {
         root: projectPath,
-        pbxproj: path.join(projectPath, `${projectName}.xcodeproj`, 'project.pbxproj')
+        pbxproj: path.join(projectPath, 'App.xcodeproj', 'project.pbxproj')
     };
 
     return projectFile.parse(locations);
@@ -85,10 +83,11 @@ function getDefaultSimulatorTarget () {
     events.emit('log', 'Select last emulator from list as default.');
     return require('./listEmulatorBuildTargets').run()
         .then(emulators => {
-            let targetEmulator;
-            if (emulators.length > 0) {
-                targetEmulator = emulators[0];
+            if (emulators.length === 0) {
+                return Promise.reject(new CordovaError('Could not find any iOS simulators. Use Xcode to install simulator devices for testing.'));
             }
+
+            let targetEmulator = emulators[0];
             emulators.forEach(emulator => {
                 if (emulator.name.indexOf('iPhone') === 0) {
                     targetEmulator = emulator;
@@ -98,49 +97,85 @@ function getDefaultSimulatorTarget () {
         });
 }
 
-/** @returns {Promise<void>} */
-module.exports.run = function (buildOpts) {
-    const projectPath = this.root;
-    let emulatorTarget = '';
-    let projectName = '';
+function parseOptions (options) {
+    options = options || {};
+    options.argv = nopt({
+        codeSignIdentity: String,
+        developmentTeam: String,
+        packageType: String,
+        provisioningProfile: String,
+        automaticProvisioning: Boolean,
+        authenticationKeyPath: String,
+        authenticationKeyID: String,
+        authenticationKeyIssuerID: String,
+        buildFlag: [String, Array],
+        iCloudContainerEnvironment: String
+    }, {}, options.argv, 0);
 
-    buildOpts = buildOpts || {};
-
-    if (buildOpts.debug && buildOpts.release) {
-        return Promise.reject(new CordovaError('Cannot specify "debug" and "release" options together.'));
+    if (options.debug && options.release) {
+        throw new CordovaError('Cannot specify "debug" and "release" options together.');
     }
 
-    if (buildOpts.device && buildOpts.emulator) {
-        return Promise.reject(new CordovaError('Cannot specify "device" and "emulator" options together.'));
+    if (options.device && options.emulator) {
+        throw new CordovaError('Cannot specify "device" and "emulator" options together.');
     }
 
-    if (buildOpts.buildConfig) {
-        if (!fs.existsSync(buildOpts.buildConfig)) {
-            return Promise.reject(new CordovaError(`Build config file does not exist: ${buildOpts.buildConfig}`));
+    buildConfigProperties.forEach(key => {
+        options[key] = options.argv[key] || options[key];
+    });
+
+    if (options.buildConfig) {
+        if (!fs.existsSync(options.buildConfig)) {
+            throw new CordovaError(`Build config file does not exist: ${options.buildConfig}`);
         }
-        events.emit('log', `Reading build config file: ${path.resolve(buildOpts.buildConfig)}`);
-        const contents = fs.readFileSync(buildOpts.buildConfig, 'utf-8');
+        events.emit('log', `Reading build config file: ${path.resolve(options.buildConfig)}`);
+        const contents = fs.readFileSync(options.buildConfig, 'utf-8');
         const buildConfig = JSON.parse(contents.replace(/^\ufeff/, '')); // Remove BOM
         if (buildConfig.ios) {
-            const buildType = buildOpts.release ? 'release' : 'debug';
+            const buildType = options.release ? 'release' : 'debug';
             const config = buildConfig.ios[buildType];
             if (config) {
                 buildConfigProperties.forEach(key => {
-                    buildOpts[key] = buildOpts[key] || config[key];
+                    options[key] = options[key] || config[key];
                 });
             }
         }
     }
 
-    return require('./listDevices').run()
-        .then(devices => {
-            if (devices.length > 0 && !(buildOpts.emulator)) {
-                // we also explicitly set device flag in options as we pass
-                // those parameters to other api (build as an example)
-                buildOpts.device = true;
-                return check_reqs.check_ios_deploy();
+    return options;
+}
+
+/** @returns {Promise<void>} */
+module.exports.run = function (buildOpts) {
+    try {
+        buildOpts = parseOptions(buildOpts);
+    } catch (e) {
+        return Promise.reject(e);
+    }
+
+    const projectPath = this.root;
+    let emulatorTarget = 'iOS Device';
+
+    if (buildOpts.target && buildOpts.target.match(/mac/i)) {
+        buildOpts.catalyst = true;
+        buildOpts.device = true;
+        buildOpts.emulator = false;
+
+        emulatorTarget = 'macOS Catalyst';
+    }
+
+    return Promise.resolve()
+        .then(() => {
+            if (!buildOpts.emulator && !buildOpts.catalyst) {
+                return require('./listDevices').run().then(devices => {
+                    if (devices.length > 0) {
+                        // we explicitly set device flag in options
+                        buildOpts.device = true;
+                    }
+                });
             }
-        }).then(() => {
+        })
+        .then(() => {
             // CB-12287: Determine the device we should target when building for a simulator
             if (!buildOpts.device) {
                 let newTarget = buildOpts.target || '';
@@ -155,7 +190,7 @@ module.exports.run = function (buildOpts) {
                     if (!theTarget) {
                         return getDefaultSimulatorTarget().then(defaultTarget => {
                             emulatorTarget = defaultTarget.name;
-                            events.emit('warn', `No simulator found for "${newTarget}. Falling back to the default target.`);
+                            events.emit('warn', `No simulator found for "${newTarget}". Falling back to the default target.`);
                             events.emit('log', `Building for "${emulatorTarget}" Simulator (${defaultTarget.identifier}, ${defaultTarget.simIdentifier}).`);
                             return emulatorTarget;
                         });
@@ -168,14 +203,12 @@ module.exports.run = function (buildOpts) {
             }
         })
         .then(() => check_reqs.run())
-        .then(() => findXCodeProjectIn(projectPath))
-        .then(name => {
-            projectName = name;
+        .then(() => {
             let extraConfig = '';
             if (buildOpts.codeSignIdentity) {
                 extraConfig += `CODE_SIGN_IDENTITY = ${buildOpts.codeSignIdentity}\n`;
-                extraConfig += `CODE_SIGN_IDENTITY[sdk=iphoneos*] = ${buildOpts.codeSignIdentity}\n`;
             }
+
             if (buildOpts.provisioningProfile) {
                 if (typeof buildOpts.provisioningProfile === 'string') {
                     extraConfig += `PROVISIONING_PROFILE_SPECIFIER = ${buildOpts.provisioningProfile}\n`;
@@ -189,7 +222,7 @@ module.exports.run = function (buildOpts) {
             }
 
             function writeCodeSignStyle (value) {
-                const project = createProjectObject(projectPath, projectName);
+                const project = createProjectObject(projectPath);
 
                 events.emit('verbose', `Set CODE_SIGN_STYLE Build Property to ${value}.`);
                 project.xcode.updateBuildProperty('CODE_SIGN_STYLE', value);
@@ -207,11 +240,11 @@ module.exports.run = function (buildOpts) {
                 writeCodeSignStyle('Automatic');
             }
 
-            return fsp.writeFile(path.join(projectPath, 'cordova', 'build-extras.xcconfig'), extraConfig, 'utf-8');
+            return fs.promises.writeFile(path.join(projectPath, 'cordova', 'build-extras.xcconfig'), extraConfig, 'utf-8');
         }).then(() => {
             const configuration = buildOpts.release ? 'Release' : 'Debug';
 
-            events.emit('log', `Building project: ${path.join(projectPath, `${projectName}.xcworkspace`)}`);
+            events.emit('log', `Building project: ${path.join(projectPath, 'App.xcworkspace')}`);
             events.emit('log', `\tConfiguration: ${configuration}`);
             events.emit('log', `\tPlatform: ${buildOpts.device ? 'device' : 'emulator'}`);
             events.emit('log', `\tTarget: ${emulatorTarget}`);
@@ -221,14 +254,14 @@ module.exports.run = function (buildOpts) {
             // remove the build output folder before building
             fs.rmSync(buildOutputDir, { recursive: true, force: true });
 
-            const xcodebuildArgs = getXcodeBuildArgs(projectName, projectPath, configuration, emulatorTarget, buildOpts);
+            const xcodebuildArgs = getXcodeBuildArgs(projectPath, configuration, emulatorTarget, buildOpts);
             return execa('xcodebuild', xcodebuildArgs, { cwd: projectPath, stdio: 'inherit' });
         }).then(() => {
-            if (!buildOpts.device || buildOpts.noSign) {
+            if (!buildOpts.device || buildOpts.catalyst || buildOpts.noSign) {
                 return;
             }
 
-            const project = createProjectObject(projectPath, projectName);
+            const project = createProjectObject(projectPath);
             const bundleIdentifier = project.getPackageName();
             const exportOptions = { ...buildOpts.exportOptions, compileBitcode: false, method: 'development' };
 
@@ -275,49 +308,26 @@ module.exports.run = function (buildOpts) {
             }
 
             function packageArchive () {
-                const xcodearchiveArgs = getXcodeArchiveArgs(projectName, projectPath, buildOutputDir, exportOptionsPath, buildOpts);
+                const xcodearchiveArgs = getXcodeArchiveArgs(projectPath, buildOutputDir, exportOptionsPath, buildOpts);
                 return execa('xcodebuild', xcodearchiveArgs, { cwd: projectPath, stdio: 'inherit' });
             }
 
-            return fsp.writeFile(exportOptionsPath, exportOptionsPlist, 'utf-8')
+            return fs.promises.writeFile(exportOptionsPath, exportOptionsPlist, 'utf-8')
                 .then(checkSystemRuby)
                 .then(packageArchive);
         })
         .then(() => {}); // resolve to undefined
 };
 
-/**
- * Searches for first XCode project in specified folder
- * @param  {String} projectPath Path where to search project
- * @return {Promise}            Promise either fulfilled with project name or rejected
- */
-function findXCodeProjectIn (projectPath) {
-    // 'Searching for Xcode project in ' + projectPath);
-    const xcodeProjFiles = fs.readdirSync(projectPath).filter(name => path.extname(name) === '.xcodeproj');
-
-    if (xcodeProjFiles.length === 0) {
-        return Promise.reject(new CordovaError(`No Xcode project found in ${projectPath}`));
-    }
-    if (xcodeProjFiles.length > 1) {
-        events.emit('warn', `Found multiple .xcodeproj directories in \n${projectPath}\nUsing first one`);
-    }
-
-    const projectName = path.basename(xcodeProjFiles[0], '.xcodeproj');
-    return Promise.resolve(projectName);
-}
-
-module.exports.findXCodeProjectIn = findXCodeProjectIn;
-
 /**
  * Returns array of arguments for xcodebuild
- * @param  {String}  projectName    Name of xcode project
  * @param  {String}  projectPath    Path to project file. Will be used to set CWD for xcodebuild
  * @param  {String}  configuration  Configuration name: debug|release
  * @param  {String}  emulatorTarget Target for emulator (rather than default)
  * @param  {Object}  buildConfig    The build configuration options
  * @return {Array}                  Array of arguments that could be passed directly to spawn method
  */
-function getXcodeBuildArgs (projectName, projectPath, configuration, emulatorTarget, buildConfig = {}) {
+function getXcodeBuildArgs (projectPath, configuration, emulatorTarget, buildConfig = {}) {
     let options;
     let buildActions;
     let settings;
@@ -335,13 +345,13 @@ function getXcodeBuildArgs (projectName, projectPath, configuration, emulatorTar
         }
     }
 
-    if (buildConfig.device) {
+    if (buildConfig.device && !buildConfig.catalyst) {
         options = [
-            '-workspace', customArgs.workspace || `${projectName}.xcworkspace`,
-            '-scheme', customArgs.scheme || projectName,
+            '-workspace', customArgs.workspace || 'App.xcworkspace',
+            '-scheme', customArgs.scheme || 'App',
             '-configuration', customArgs.configuration || configuration,
             '-destination', customArgs.destination || 'generic/platform=iOS',
-            '-archivePath', customArgs.archivePath || `${projectName}.xcarchive`
+            '-archivePath', customArgs.archivePath || 'App.xcarchive'
         ];
         buildActions = ['archive'];
         settings = [];
@@ -374,14 +384,24 @@ function getXcodeBuildArgs (projectName, projectPath, configuration, emulatorTar
         }
     } else { // emulator
         options = [
-            '-workspace', customArgs.workspace || `${projectName}.xcworkspace`,
-            '-scheme', customArgs.scheme || projectName,
-            '-configuration', customArgs.configuration || configuration,
-            '-sdk', customArgs.sdk || 'iphonesimulator',
-            '-destination', customArgs.destination || `platform=iOS Simulator,name=${emulatorTarget}`
+            '-workspace', customArgs.workspace || 'App.xcworkspace',
+            '-scheme', customArgs.scheme || 'App',
+            '-configuration', customArgs.configuration || configuration
         ];
+
+        if (buildConfig.catalyst) {
+            options = options.concat([
+                '-destination', customArgs.destination || 'generic/platform=macOS,variant=Mac Catalyst'
+            ]);
+        } else {
+            options = options.concat([
+                '-sdk', customArgs.sdk || 'iphonesimulator',
+                '-destination', customArgs.destination || `platform=iOS Simulator,name=${emulatorTarget}`
+            ]);
+        }
+
         buildActions = ['build'];
-        settings = [`SYMROOT=${path.join(projectPath, 'build')}`];
+        settings = [];
 
         if (customArgs.configuration_build_dir) {
             settings.push(customArgs.configuration_build_dir);
@@ -403,15 +423,27 @@ function getXcodeBuildArgs (projectName, projectPath, configuration, emulatorTar
 
 /**
  * Returns array of arguments for xcodebuild
- * @param  {String}  projectName        Name of xcode project
  * @param  {String}  projectPath        Path to project file. Will be used to set CWD for xcodebuild
  * @param  {String}  outputPath         Output directory to contain the IPA
  * @param  {String}  exportOptionsPath  Path to the exportOptions.plist file
  * @param  {Object}  buildConfig        Build configuration options
  * @return {Array}                      Array of arguments that could be passed directly to spawn method
  */
-function getXcodeArchiveArgs (projectName, projectPath, outputPath, exportOptionsPath, buildConfig = {}) {
+function getXcodeArchiveArgs (projectPath, outputPath, exportOptionsPath, buildConfig = {}) {
     const options = [];
+    const buildFlags = buildConfig.buildFlag;
+    const customArgs = {};
+    customArgs.otherFlags = [];
+
+    if (buildFlags) {
+        if (typeof buildFlags === 'string' || buildFlags instanceof String) {
+            parseBuildFlag(buildFlags, customArgs);
+        } else { // buildFlags is an Array of strings
+            buildFlags.forEach(flag => {
+                parseBuildFlag(flag, customArgs);
+            });
+        }
+    }
 
     if (buildConfig.automaticProvisioning) {
         options.push('-allowProvisioningUpdates');
@@ -428,10 +460,10 @@ function getXcodeArchiveArgs (projectName, projectPath, outputPath, exportOption
 
     return [
         '-exportArchive',
-        '-archivePath', `${projectName}.xcarchive`,
+        '-archivePath', customArgs.archivePath || 'App.xcarchive',
         '-exportOptionsPlist', exportOptionsPath,
         '-exportPath', outputPath
-    ].concat(options);
+    ].concat(options).concat(customArgs.otherFlags);
 }
 
 function parseBuildFlag (buildFlag, args) {
@@ -442,7 +474,7 @@ function parseBuildFlag (buildFlag, args) {
             matched = true;
             // found[0] is the whole match, found[1] is the first match in parentheses.
             args[key] = found[1];
-            events.emit('warn', util.format('Overriding xcodebuildArg: %s', buildFlag));
+            events.emit('warn', `Overriding xcodebuildArg: ${buildFlag}`);
         }
     }
 
@@ -450,13 +482,12 @@ function parseBuildFlag (buildFlag, args) {
         // If the flag starts with a '-' then it is an xcodebuild built-in option or a
         // user-defined setting. The regex makes sure that we don't split a user-defined
         // setting that is wrapped in quotes.
-        /* eslint-disable no-useless-escape */
-        if (buildFlag[0] === '-' && !buildFlag.match(/^.*=(\".*\")|(\'.*\')$/)) {
+        if (buildFlag[0] === '-' && !buildFlag.match(/^[^=]+=(["'])(.*?[^\\])\1$/)) {
             args.otherFlags = args.otherFlags.concat(buildFlag.split(' '));
-            events.emit('warn', util.format('Adding xcodebuildArg: %s', buildFlag.split(' ')));
+            events.emit('warn', `Adding xcodebuildArg: ${buildFlag.split(' ')}`);
         } else {
             args.otherFlags.push(buildFlag);
-            events.emit('warn', util.format('Adding xcodebuildArg: %s', buildFlag));
+            events.emit('warn', `Adding xcodebuildArg: ${buildFlag}`);
         }
     }
 }
