@@ -20,21 +20,15 @@
 const path = require('node:path');
 const fs = require('node:fs');
 const execa = require('execa');
-const { CordovaError } = require('cordova-common');
 
 module.exports.run = function () {
     const projectPath = this.root;
-    const projectName = fs.readdirSync(projectPath).filter(name => path.extname(name) === '.xcodeproj');
-
-    if (!projectName) {
-        return Promise.reject(new CordovaError(`No Xcode project found in ${projectPath}`));
-    }
 
     const xcodebuildClean = configName => {
         return execa(
             'xcodebuild',
-            ['-project', projectName, '-configuration', configName, '-alltargets', 'clean'],
-            { cwd: projectPath, stdio: 'inherit' }
+            ['-project', 'App.xcodeproj', '-configuration', configName, '-alltargets', 'clean'],
+            { cwd: this.root, stdio: 'inherit' }
         );
     };
 
