@@ -17,9 +17,9 @@
 'use strict';
 const fs = require('node:fs');
 const path = require('node:path');
-const util = require('node:util');
 const events = require('cordova-common').events;
 const CordovaError = require('cordova-common').CordovaError;
+const { isSwiftPackagePlugin } = require('../SwiftPackage');
 
 // These frameworks are required by cordova-ios by default. We should never add/remove them.
 const keep_these_frameworks = [
@@ -31,22 +31,32 @@ const keep_these_frameworks = [
 const handlers = {
     'source-file': {
         install: function (obj, plugin, project, options) {
+            if (isSwiftPackagePlugin(plugin)) return;
+
             installHelper('source-file', obj, plugin.dir, project.projectDir, plugin.id, options, project);
         },
         uninstall: function (obj, plugin, project, options) {
+            if (isSwiftPackagePlugin(plugin)) return;
+
             uninstallHelper('source-file', obj, project.projectDir, plugin.id, options, project);
         }
     },
     'header-file': {
         install: function (obj, plugin, project, options) {
+            if (isSwiftPackagePlugin(plugin)) return;
+
             installHelper('header-file', obj, plugin.dir, project.projectDir, plugin.id, options, project);
         },
         uninstall: function (obj, plugin, project, options) {
+            if (isSwiftPackagePlugin(plugin)) return;
+
             uninstallHelper('header-file', obj, project.projectDir, plugin.id, options, project);
         }
     },
     'resource-file': {
         install: function (obj, plugin, project, options) {
+            if (isSwiftPackagePlugin(plugin)) return;
+
             const src = obj.src;
             let target = obj.target;
             const srcFile = path.resolve(plugin.dir, src);
@@ -67,6 +77,8 @@ const handlers = {
             copyFile(plugin.dir, src, project.projectDir, destFile, link);
         },
         uninstall: function (obj, plugin, project, options) {
+            if (isSwiftPackagePlugin(plugin)) return;
+
             const src = obj.src;
             let target = obj.target;
 
@@ -81,6 +93,8 @@ const handlers = {
     },
     framework: { // CB-5238 custom frameworks only
         install: function (obj, plugin, project, options) {
+            if (isSwiftPackagePlugin(plugin)) return;
+
             const src = obj.src;
             const custom = !!(obj.custom); // convert to boolean (if truthy/falsy)
             const embed = !!(obj.embed); // convert to boolean (if truthy/falsy)
@@ -102,9 +116,9 @@ const handlers = {
                         project.frameworks[src] = project.frameworks[src] || 0;
                         project.frameworks[src]++;
                         const opt = { customFramework: false, embed: false, link: true, weak: obj.weak };
-                        events.emit('verbose', util.format('Adding non-custom framework to project... %s -> %s', src, JSON.stringify(opt)));
+                        events.emit('verbose', `Adding non-custom framework to project... ${src} -> ${JSON.stringify(opt)}`);
                         project.xcode.addFramework(src, opt);
-                        events.emit('verbose', util.format('Non-custom framework added to project. %s -> %s', src, JSON.stringify(opt)));
+                        events.emit('verbose', `Non-custom framework added to project. ${src} -> ${JSON.stringify(opt)}`);
                     }
                 }
                 return;
@@ -124,11 +138,13 @@ const handlers = {
                 project.xcode.addBuildPhase([], 'PBXCopyFilesBuildPhase', 'Embed Frameworks', null, 'frameworks');
             }
             const opt = { customFramework: true, embed, link, sign: true };
-            events.emit('verbose', util.format('Adding custom framework to project... %s -> %s', src, JSON.stringify(opt)));
+            events.emit('verbose', `Adding custom framework to project... ${src} -> ${JSON.stringify(opt)}`);
             project.xcode.addFramework(project_relative, opt);
-            events.emit('verbose', util.format('Custom framework added to project. %s -> %s', src, JSON.stringify(opt)));
+            events.emit('verbose', `Custom framework added to project. ${src} -> ${JSON.stringify(opt)}`);
         },
         uninstall: function (obj, plugin, project, options) {
+            if (isSwiftPackagePlugin(plugin)) return;
+
             const src = obj.src;
 
             if (!obj.custom) { // CB-9825 cocoapod integration for plugins
@@ -167,6 +183,8 @@ const handlers = {
     },
     asset: {
         install: function (obj, plugin, project, options) {
+            if (isSwiftPackagePlugin(plugin)) return;
+
             if (!obj.src) {
                 throw new CordovaError(generateAttributeError('src', 'asset', plugin.id));
             }
@@ -178,17 +196,17 @@ const handlers = {
             if (options && options.usePlatformWww) copyFile(plugin.dir, obj.src, project.platformWww, obj.target);
         },
         uninstall: function (obj, plugin, project, options) {
+            if (isSwiftPackagePlugin(plugin)) return;
+
             const target = obj.target;
 
             if (!target) {
                 throw new CordovaError(generateAttributeError('target', 'asset', plugin.id));
             }
 
-            removeFile(project.www, target);
-            removeFileF(path.resolve(project.www, 'plugins', plugin.id));
+            removeFileAndParents(project.www, target);
             if (options && options.usePlatformWww) {
-                removeFile(project.platformWww, target);
-                removeFileF(path.resolve(project.platformWww, 'plugins', plugin.id));
+                removeFileAndParents(project.platformWww, target);
             }
         }
     },
@@ -305,9 +323,8 @@ function uninstallHelper (type, obj, project_dir, plugin_id, options, project) {
     }
 }
 
-const pathSepFix = new RegExp(path.sep.replace(/\\/, '\\\\'), 'g');
 function fixPathSep (file) {
-    return file.replace(pathSepFix, '/');
+    return file.replaceAll(path.sep, path.posix.sep);
 }
 
 function copyFile (plugin_dir, src, project_dir, dest, link) {
@@ -356,15 +373,9 @@ function linkFileOrDirTree (src, dest) {
     }
 }
 
-// checks if file exists and then deletes. Error if doesn't exist
-function removeFile (project_dir, src) {
-    const file = path.resolve(project_dir, src);
-    fs.rmSync(file);
-}
-
 // deletes file/directory without checking
 function removeFileF (file) {
-    fs.rmSync(file);
+    fs.rmSync(file, { recursive: true, force: true });
 }
 
 function removeFileAndParents (baseDir, destFile, stopper) {
