@@ -20,7 +20,6 @@
 const fs = require('node:fs');
 const path = require('node:path');
 const xcode = require('xcode');
-const plist = require('plist');
 
 const pluginHandlers = require('./plugman/pluginHandlers');
 const CordovaError = require('cordova-common').CordovaError;
@@ -38,13 +37,10 @@ function parseProjectFile (locations) {
     const xcodeproj = xcode.project(pbxPath);
     xcodeproj.parseSync();
 
-    const xcBuildConfiguration = xcodeproj.pbxXCBuildConfigurationSection();
-    const plist_file_entry = Object.values(xcBuildConfiguration).find(entry => entry.buildSettings && entry.buildSettings.INFOPLIST_FILE);
-    const plist_file = path.join(project_dir, plist_file_entry.buildSettings.INFOPLIST_FILE.replace(/^"(.*)"$/g, '$1').replace(/\\&/g, '&'));
-    const config_file = path.join(path.dirname(plist_file), 'config.xml');
+    const config_file = path.join(project_dir, 'App', 'config.xml');
 
-    if (!fs.existsSync(plist_file) || !fs.existsSync(config_file)) {
-        throw new CordovaError('Could not find *-Info.plist file, or config.xml file.');
+    if (!fs.existsSync(config_file)) {
+        throw new CordovaError('Could not find config.xml file.');
     }
 
     const frameworks_file = path.join(project_dir, 'frameworks.json');
@@ -53,7 +49,7 @@ function parseProjectFile (locations) {
         frameworks = require(frameworks_file);
     } catch (e) { }
 
-    const xcode_dir = path.dirname(plist_file);
+    const xcode_dir = path.join(project_dir, 'App');
     const pluginsDir = path.resolve(xcode_dir, 'Plugins');
     const resourcesDir = path.resolve(xcode_dir, 'Resources');
 
@@ -67,7 +63,7 @@ function parseProjectFile (locations) {
         platformWww: path.join(project_dir, 'platform_www'),
         www: path.join(project_dir, 'www'),
         write: function () {
-            fs.writeFileSync(pbxPath, xcodeproj.writeSync());
+            fs.writeFileSync(pbxPath, xcodeproj.writeSync({ omitEmptyValues: true }));
             if (Object.keys(this.frameworks).length === 0) {
                 // If there is no framework references remain in the project, just remove this file
                 fs.rmSync(frameworks_file, { force: true });
@@ -76,15 +72,7 @@ function parseProjectFile (locations) {
             fs.writeFileSync(frameworks_file, JSON.stringify(this.frameworks, null, 4));
         },
         getPackageName: function () {
-            const packageName = plist.parse(fs.readFileSync(plist_file, 'utf8')).CFBundleIdentifier;
-            let bundleIdentifier = packageName;
-
-            const variables = packageName.match(/\$\((\w+)\)/); // match $(VARIABLE), if any
-            if (variables && variables.length >= 2) {
-                bundleIdentifier = xcodeproj.getBuildProperty(variables[1]);
-            }
-
-            return bundleIdentifier.replace(/^"/, '').replace(/"$/, '');
+            return xcodeproj.getBuildProperty('PRODUCT_BUNDLE_IDENTIFIER', undefined, 'App').replace(/^"/, '').replace(/"$/, '');
         },
         getInstaller: function (name) {
             return pluginHandlers.getInstaller(name);
@@ -125,7 +113,6 @@ xcode.project.prototype.removeFromPbxEmbedFrameworksBuildPhase = function (file)
 
 // special handlers to add frameworks to the 'Embed Frameworks' build phase, needed for custom frameworks
 // see CB-9517. should probably be moved to node-xcode.
-const util = require('util');
 function pbxBuildPhaseObj (file) {
     const obj = Object.create(null);
     obj.value = file.uuid;
@@ -134,5 +121,5 @@ function pbxBuildPhaseObj (file) {
 }
 
 function longComment (file) {
-    return util.format('%s in %s', file.basename, file.group);
+    return `${file.basename} in ${file.group}`;
 }
